
# line 2 "S3Parser.y"
#include "s3dt.h"
#include "s3dtint.h"

static int reservedword(char *s);
int s3yyparse(void);
static int s3yylex(void);
static void nextchar(void);
static void parseerror(char *s1, char *s2);
static void s3yyerror(char *s);

static int parse_debug = 0;

#define S3YYMAXDEPTH 1000

# line 17 "S3Parser.y"
typedef union
#ifdef __cplusplus
	S3YYSTYPE
#endif
 {
	double num;
	char string[256];
	} S3YYSTYPE;
# define NUM 257
# define STRING 258
# define OBJECT 259
# define ID 260
# define NAME 261
# define RADIUS 262
# define POS 263
# define POINTS 264
# define FACE 265
# define END 266
# define EXTRA 267

#include <malloc.h>
#include <memory.h>
#include <unistd.h>
#include <values.h>

#ifdef __cplusplus

#ifndef s3yyerror
	void s3yyerror(const char *);
#endif
#ifndef s3yylex
	extern "C" int s3yylex(void);
#endif
	int s3yyparse(void);

#endif
#define s3yyclearin s3yychar = -1
#define s3yyerrok s3yyerrflag = 0
extern int s3yychar;
extern int s3yyerrflag;
S3YYSTYPE s3yylval;
S3YYSTYPE s3yyval;
typedef int s3yytabelem;
#ifndef S3YYMAXDEPTH
#define S3YYMAXDEPTH 150
#endif
#if S3YYMAXDEPTH > 0
int s3yy_s3yys[S3YYMAXDEPTH], *s3yys = s3yy_s3yys;
S3YYSTYPE s3yy_s3yyv[S3YYMAXDEPTH], *s3yyv = s3yy_s3yyv;
#else	/* user does initial allocation */
int *s3yys;
S3YYSTYPE *s3yyv;
#endif
static int s3yymaxdepth = S3YYMAXDEPTH;
# define S3YYERRCODE 256

# line 174 "S3Parser.y"

static S3object *no;			/* object currently being read */
static S3polygon *np;			/* polygon currently being read */
static S3objtab *oh;			/* table to add objects to */
static char currch;			/* character being processed */
static char filename[256];		/* file being read */
static int lineno;			/* line number up to in file */
static int vert_tmp[MAX_VERTS_PER_FACE];/* space for new polygon vertices */
static int fd;				/* file descriptor to read from */
static char *evlist[30];		/* temp space for extra data attribs */
static int evcount;

/* s3yyerror
 * Function called by yacc when an error occurs in parsing
 */
static void s3yyerror(char *s)
{
parseerror("parse error",s);
}


/* parseerror
 * Function called when a semantic error occurs in the input data
 */
static void parseerror(char *s1, char *s2)
{
if (*s2)
	fprintf(stderr,"%s : %s\nNear line %d in %s\n",s1,s2,lineno,filename);
else
	fprintf(stderr,"%s\nNear line %d in %s\n",s1,s2,lineno,filename);
}


/* nextchar
 * Read one character from the input fd
 */
static void nextchar(void)
{
read(fd,&currch,1);
if (currch == '\n')
	lineno++;
}


/* s3yylex
 * Read one token
 */
static int s3yylex(void)
{
char tok[256] = {0};
char *tp = tok;

/* skip whitespace
 */
while(currch == ' ' || currch == '\t' || currch == '\n') {
	nextchar();
	}

if (isdigit(currch) || currch == '.') {
	/* A number
	 */
	double num;

	while(isdigit(currch) || currch == '.') {
		*tp++ = currch;
		nextchar();
		}
	s3yylval.num = atof(tok);
	if (parse_debug)
		printf("num : %f\n",s3yylval.num);
	return NUM;
	}
else if (currch == '\"') {
	/* A quoted string
	 */
	nextchar();
	while(currch != '\"') {
		*tp++ = currch;
		nextchar();
		}
	nextchar();
	strcpy(s3yylval.string,tok);
	if (parse_debug)
		printf("string : %s\n",s3yylval.string);
	return STRING;
	}
else if (isalpha(currch)) {
	/* An alphanumeric string
	 */
	int rw;

	while(isdigit(currch) || isalpha(currch) || currch == '_') {
		*tp++ = currch;
		nextchar();
		}
	if ((rw = reservedword(tok)) != -1) {
		if (parse_debug)
			printf("reserved : %d = %s\n",rw,tok);
		return rw;
		}
	strcpy(s3yylval.string,tok);
	if (parse_debug)
		printf("string : %s\n",s3yylval.string);
	return STRING;
	}
else if (currch == '#') {
	/* cpp file info line
	 */
	char line[256];		/* line starting with '#' */
	char *lnum;		/* real line number */
	char *fnam;		/* real file name */
	int i=0;

	while(currch != '\n') {
		nextchar();
		line[i++] = currch;
		}
	lnum = strtok(line," \n");
	fnam = strtok(NULL," \n");
	lineno = atoi(lnum);
	strcpy(filename,fnam);
	nextchar();
	return s3yylex();		/* call again to get next token */
	}
else if (currch == -1) {
	/* end of file
	 */
	return END;
	}
else {
	/* some other character.
	 */
	int ch = currch;

	nextchar();
	if (parse_debug)
		printf("char : %c\n",ch);
	return ch;
	}
}


/* reservedword
 * Return the token for reserved word, or -1 if no match
 */
static int reservedword(char *s)
{
int i;
struct {
	char *word;
	int tok;
	} rword[] = {	"object",OBJECT,"id",ID,"name",NAME,"radius",RADIUS,
			"pos",POS,"points",POINTS,"face",FACE,"end",END,
			"extra",EXTRA };

for(i=0; i<sizeof(rword)/sizeof(rword[0]); i++)
	if (!strcmp(rword[i].word,s))
		return rword[i].tok;
return -1;
}


/* S3ParseFd
 * Parse an object description file from a file descriptor.
 * Returns 0 if the parse was successful, or 1 if not.
 */
int S3ParseFd(int filed, S3objtab *ohead)
{
oh = ohead;
fd = filed;
currch = ' ';
strcpy(filename,"unknown");
lineno = 0;
return s3yyparse();
}


/* S3ParseFile
 * Opens a file and reads s3dt object descriptions from it.
 * Returns 0 if successful, or 1 if an error occurs.
 */
int S3ParseFile(char *fname, S3objtab *ohead)
{
if ((fd = open(fname,O_RDONLY)) == -1)
	return 1;
oh = ohead;
currch = ' ';
strcpy(filename,"fname");
return s3yyparse();
}

s3yytabelem s3yyexca[] ={
-1, 1,
	0, -1,
	-2, 0,
	};
# define S3YYNPROD 32
# define S3YYLAST 202
s3yytabelem s3yyact[]={

    10,     5,    49,    39,    19,    29,     6,    19,     3,    45,
    38,    32,    24,    22,     7,    18,    50,    48,    44,    40,
    46,    33,    36,    30,    43,    35,    26,    11,     9,    20,
    21,     8,    23,     4,     2,     1,    27,    25,     0,    28,
     0,    31,    34,     0,     0,     0,     0,     0,     0,     0,
    37,     0,     0,    42,     0,     0,     0,     0,     0,    47,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,    12,    41,    13,    14,    15,
    16,    19,    17,    19,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,    51 };
s3yytabelem s3yypact[]={

-10000000,-10000000,  -258,-10000000,-10000000,  -252,  -109,-10000000,-10000000,  -125,
-10000000,-10000000,  -250,  -250,  -250,  -110,  -250,  -111,-10000000,-10000000,
-10000000,  -250,-10000000,  -253,-10000000,  -250,  -114,-10000000,-10000000,-10000000,
-10000000,-10000000,-10000000,-10000000,  -250,  -113,  -122,  -250,-10000000,-10000000,
-10000000,-10000000,-10000000,  -116,-10000000,-10000000,-10000000,-10000000,   -57,-10000000,
-10000000,-10000000 };
s3yytabelem s3yypgo[]={

     0,    15,    36,    35,    34,    33,    31,    28,    27,    26,
    25,    24,    23,    22,    21,    20,    19,    18,    17,    16 };
s3yytabelem s3yyr1[]={

     0,     3,     4,     4,     6,     5,     7,     7,     8,     8,
     8,     8,    10,     8,    12,     8,     9,     9,    14,    11,
    11,    15,     1,     2,     2,    13,    13,    17,    16,    18,
    18,    19 };
s3yytabelem s3yyr2[]={

     0,     5,     4,     0,     1,    12,     4,     0,     5,     5,
     9,     8,     1,    15,     1,    10,     4,     0,     7,     4,
     0,     3,     2,     3,     3,     4,     0,     1,     9,     4,
     0,     3 };
s3yytabelem s3yychk[]={

-10000000,    -3,    -4,   266,    -5,   259,   258,   123,    -6,    -7,
   125,    -8,   260,   262,   263,   264,   265,   267,    -1,   257,
    -1,    -1,   123,    -1,   123,    -1,    -9,    -2,    -1,   258,
   -12,    -1,   125,   -14,    -1,   -10,   -13,    -1,   123,   125,
   -16,   258,    -1,   -11,   -17,   125,   -15,    -1,   -18,    59,
   -19,   258 };
s3yytabelem s3yydef[]={

     3,    -2,     0,     1,     2,     0,     0,     4,     7,     0,
     5,     6,     0,     0,     0,     0,     0,     0,     8,    22,
     9,     0,    17,     0,    14,     0,     0,    12,    23,    24,
    26,    10,    11,    16,     0,     0,     0,     0,    20,    15,
    25,    27,    18,     0,    30,    13,    19,    21,     0,    28,
    29,    31 };
typedef struct
#ifdef __cplusplus
	s3yytoktype
#endif
{ char *t_name; int t_val; } s3yytoktype;
#ifndef S3YYDEBUG
#	define S3YYDEBUG	0	/* don't allow debugging */
#endif

#if S3YYDEBUG

s3yytoktype s3yytoks[] =
{
	"NUM",	257,
	"STRING",	258,
	"OBJECT",	259,
	"ID",	260,
	"NAME",	261,
	"RADIUS",	262,
	"POS",	263,
	"POINTS",	264,
	"FACE",	265,
	"END",	266,
	"EXTRA",	267,
	"-unknown-",	-1	/* ends search */
};

char * s3yyreds[] =
{
	"-no such reduction-",
	"file : lines END",
	"lines : lines object",
	"lines : /* empty */",
	"object : OBJECT STRING '{'",
	"object : OBJECT STRING '{' objattribs '}'",
	"objattribs : objattribs objattrib",
	"objattribs : /* empty */",
	"objattrib : ID expr",
	"objattrib : RADIUS expr",
	"objattrib : POS expr expr expr",
	"objattrib : POINTS '{' coords '}'",
	"objattrib : FACE expr facetype",
	"objattrib : FACE expr facetype '{' vertices '}'",
	"objattrib : EXTRA '{'",
	"objattrib : EXTRA '{' extraattribs '}'",
	"coords : coords coord",
	"coords : /* empty */",
	"coord : expr expr expr",
	"vertices : vertices vertex",
	"vertices : /* empty */",
	"vertex : expr",
	"expr : NUM",
	"facetype : expr",
	"facetype : STRING",
	"extraattribs : extraattribs extraattrib",
	"extraattribs : /* empty */",
	"extraattrib : STRING",
	"extraattrib : STRING extravals ';'",
	"extravals : extravals extraval",
	"extravals : /* empty */",
	"extraval : STRING",
};
#endif /* S3YYDEBUG */
/* 
 *	Copyright 1987 Silicon Graphics, Inc. - All Rights Reserved
 */

/* #ident	"@(#)yacc:yaccpar	1.10" */
#ident	"$Revision: 1.9 $"

/*
** Skeleton parser driver for yacc output
*/

/*
** yacc user known macros and defines
*/
#define S3YYERROR		goto s3yyerrlab
#define S3YYACCEPT	return(0)
#define S3YYABORT		return(1)
#ifdef __cplusplus
#define S3YYBACKUP( newtoken, newvalue )\
{\
	if ( s3yychar >= 0 || ( s3yyr2[ s3yytmp ] >> 1 ) != 1 )\
	{\
		s3yyerror( gettxt("uxlibc:78", "syntax error - cannot backup") );\
		goto s3yyerrlab;\
	}\
	s3yychar = newtoken;\
	s3yystate = *s3yyps;\
	s3yylval = newvalue;\
	goto s3yynewstate;\
}
#else
#define S3YYBACKUP( newtoken, newvalue )\
{\
	if ( s3yychar >= 0 || ( s3yyr2[ s3yytmp ] >> 1 ) != 1 )\
	{\
		s3yyerror( gettxt("uxlibc:78", "Syntax error - cannot backup") );\
		goto s3yyerrlab;\
	}\
	s3yychar = newtoken;\
	s3yystate = *s3yyps;\
	s3yylval = newvalue;\
	goto s3yynewstate;\
}
#endif
#define S3YYRECOVERING()	(!!s3yyerrflag)
#define S3YYNEW(type)	malloc(sizeof(type) * s3yynewmax)
#define S3YYCOPY(to, from, type) \
	(type *) memcpy(to, (char *) from, s3yynewmax * sizeof(type))
#define S3YYENLARGE( from, type) \
	(type *) realloc((char *) from, s3yynewmax * sizeof(type))
#ifndef S3YYDEBUG
#	define S3YYDEBUG	1	/* make debugging available */
#endif

/*
** user known globals
*/
int s3yydebug;			/* set to 1 to get debugging */

/*
** driver internal defines
*/
#define S3YYFLAG		(-10000000)

/*
** global variables used by the parser
*/
S3YYSTYPE *s3yypv;			/* top of value stack */
int *s3yyps;			/* top of state stack */

int s3yystate;			/* current state */
int s3yytmp;			/* extra var (lasts between blocks) */

int s3yynerrs;			/* number of errors */
int s3yyerrflag;			/* error recovery flag */
int s3yychar;			/* current input token number */



/*
** s3yyparse - return 0 if worked, 1 if syntax error not recovered from
*/
#if defined(__STDC__) || defined(__cplusplus)
int s3yyparse(void)
#else
int s3yyparse()
#endif
{
	register S3YYSTYPE *s3yypvt;	/* top of value stack for $vars */

	/*
	** Initialize externals - s3yyparse may be called more than once
	*/
	s3yypv = &s3yyv[-1];
	s3yyps = &s3yys[-1];
	s3yystate = 0;
	s3yytmp = 0;
	s3yynerrs = 0;
	s3yyerrflag = 0;
	s3yychar = -1;

#if S3YYMAXDEPTH <= 0
	if (s3yymaxdepth <= 0)
	{
		if ((s3yymaxdepth = S3YYEXPAND(0)) <= 0)
		{
#ifdef __cplusplus
			s3yyerror(gettxt("uxlibc:79", "yacc initialization error"));
#else
			s3yyerror(gettxt("uxlibc:79", "Yacc initialization error"));
#endif
			S3YYABORT;
		}
	}
#endif

	goto s3yystack;
	{
		register S3YYSTYPE *s3yy_pv;	/* top of value stack */
		register int *s3yy_ps;		/* top of state stack */
		register int s3yy_state;		/* current state */
		register int  s3yy_n;		/* internal state number info */

		/*
		** get globals into registers.
		** branch to here only if S3YYBACKUP was called.
		*/
	s3yynewstate:
		s3yy_pv = s3yypv;
		s3yy_ps = s3yyps;
		s3yy_state = s3yystate;
		goto s3yy_newstate;

		/*
		** get globals into registers.
		** either we just started, or we just finished a reduction
		*/
	s3yystack:
		s3yy_pv = s3yypv;
		s3yy_ps = s3yyps;
		s3yy_state = s3yystate;

		/*
		** top of for (;;) loop while no reductions done
		*/
	s3yy_stack:
		/*
		** put a state and value onto the stacks
		*/
#if S3YYDEBUG
		/*
		** if debugging, look up token value in list of value vs.
		** name pairs.  0 and negative (-1) are special values.
		** Note: linear search is used since time is not a real
		** consideration while debugging.
		*/
		if ( s3yydebug )
		{
			register int s3yy_i;

			printf( "State %d, token ", s3yy_state );
			if ( s3yychar == 0 )
				printf( "end-of-file\n" );
			else if ( s3yychar < 0 )
				printf( "-none-\n" );
			else
			{
				for ( s3yy_i = 0; s3yytoks[s3yy_i].t_val >= 0;
					s3yy_i++ )
				{
					if ( s3yytoks[s3yy_i].t_val == s3yychar )
						break;
				}
				printf( "%s\n", s3yytoks[s3yy_i].t_name );
			}
		}
#endif /* S3YYDEBUG */
		if ( ++s3yy_ps >= &s3yys[ s3yymaxdepth ] )	/* room on stack? */
		{
			int s3yynewmax, s3yys_off;

			/* The following pointer-differences are safe, since
			 * s3yypvt, s3yy_pv, and s3yypv all are a multiple of
			 * sizeof(S3YYSTYPE) bytes from s3yyv.
			 */
			int s3yypvt_off = s3yypvt - s3yyv;
			int s3yy_pv_off = s3yy_pv - s3yyv;
			int s3yypv_off = s3yypv - s3yyv;

			int *s3yys_base = s3yys;
#ifdef S3YYEXPAND
			s3yynewmax = S3YYEXPAND(s3yymaxdepth);
#else
			s3yynewmax = 2 * s3yymaxdepth;	/* double table size */
			if (s3yymaxdepth == S3YYMAXDEPTH)	/* first time growth */
			{
				void *news3yys = S3YYNEW(int);
				void *news3yyv = S3YYNEW(S3YYSTYPE);
				if (news3yys != 0 && news3yyv != 0)
				{
					s3yys = S3YYCOPY(news3yys, s3yys, int);
					s3yyv = S3YYCOPY(news3yyv, s3yyv, S3YYSTYPE);
				}
				else
					s3yynewmax = 0;	/* failed */
			}
			else				/* not first time */
			{
				s3yys = S3YYENLARGE(s3yys, int);
				s3yyv = S3YYENLARGE(s3yyv, S3YYSTYPE);
				if (s3yys == 0 || s3yyv == 0)
					s3yynewmax = 0;	/* failed */
			}
#endif
			if (s3yynewmax <= s3yymaxdepth)	/* tables not expanded */
			{
#ifdef __cplusplus
				s3yyerror( gettxt("uxlibc:80", "yacc stack overflow") );
#else
				s3yyerror( gettxt("uxlibc:80", "Yacc stack overflow") );
#endif
				S3YYABORT;
			}
			s3yymaxdepth = s3yynewmax;

			/* reset pointers into s3yys */
			s3yys_off = s3yys - s3yys_base;
			s3yy_ps = s3yy_ps + s3yys_off;
			s3yyps = s3yyps + s3yys_off;

			/* reset pointers into s3yyv */
			s3yypvt = s3yyv + s3yypvt_off;
			s3yy_pv = s3yyv + s3yy_pv_off;
			s3yypv = s3yyv + s3yypv_off;
		}
		*s3yy_ps = s3yy_state;
		*++s3yy_pv = s3yyval;

		/*
		** we have a new state - find out what to do
		*/
	s3yy_newstate:
		if ( ( s3yy_n = s3yypact[ s3yy_state ] ) <= S3YYFLAG )
			goto s3yydefault;		/* simple state */
#if S3YYDEBUG
		/*
		** if debugging, need to mark whether new token grabbed
		*/
		s3yytmp = s3yychar < 0;
#endif
		if ( ( s3yychar < 0 ) && ( ( s3yychar = s3yylex() ) < 0 ) )
			s3yychar = 0;		/* reached EOF */
#if S3YYDEBUG
		if ( s3yydebug && s3yytmp )
		{
			register int s3yy_i;

			printf( "Received token " );
			if ( s3yychar == 0 )
				printf( "end-of-file\n" );
			else if ( s3yychar < 0 )
				printf( "-none-\n" );
			else
			{
				for ( s3yy_i = 0; s3yytoks[s3yy_i].t_val >= 0;
					s3yy_i++ )
				{
					if ( s3yytoks[s3yy_i].t_val == s3yychar )
						break;
				}
				printf( "%s\n", s3yytoks[s3yy_i].t_name );
			}
		}
#endif /* S3YYDEBUG */
		if ( ( ( s3yy_n += s3yychar ) < 0 ) || ( s3yy_n >= S3YYLAST ) )
			goto s3yydefault;
		if ( s3yychk[ s3yy_n = s3yyact[ s3yy_n ] ] == s3yychar )	/*valid shift*/
		{
			s3yychar = -1;
			s3yyval = s3yylval;
			s3yy_state = s3yy_n;
			if ( s3yyerrflag > 0 )
				s3yyerrflag--;
			goto s3yy_stack;
		}

	s3yydefault:
		if ( ( s3yy_n = s3yydef[ s3yy_state ] ) == -2 )
		{
#if S3YYDEBUG
			s3yytmp = s3yychar < 0;
#endif
			if ( ( s3yychar < 0 ) && ( ( s3yychar = s3yylex() ) < 0 ) )
				s3yychar = 0;		/* reached EOF */
#if S3YYDEBUG
			if ( s3yydebug && s3yytmp )
			{
				register int s3yy_i;

				printf( "Received token " );
				if ( s3yychar == 0 )
					printf( "end-of-file\n" );
				else if ( s3yychar < 0 )
					printf( "-none-\n" );
				else
				{
					for ( s3yy_i = 0;
						s3yytoks[s3yy_i].t_val >= 0;
						s3yy_i++ )
					{
						if ( s3yytoks[s3yy_i].t_val
							== s3yychar )
						{
							break;
						}
					}
					printf( "%s\n", s3yytoks[s3yy_i].t_name );
				}
			}
#endif /* S3YYDEBUG */
			/*
			** look through exception table
			*/
			{
				register int *s3yyxi = s3yyexca;

				while ( ( *s3yyxi != -1 ) ||
					( s3yyxi[1] != s3yy_state ) )
				{
					s3yyxi += 2;
				}
				while ( ( *(s3yyxi += 2) >= 0 ) &&
					( *s3yyxi != s3yychar ) )
					;
				if ( ( s3yy_n = s3yyxi[1] ) < 0 )
					S3YYACCEPT;
			}
		}

		/*
		** check for syntax error
		*/
		if ( s3yy_n == 0 )	/* have an error */
		{
			/* no worry about speed here! */
			switch ( s3yyerrflag )
			{
			case 0:		/* new error */
#ifdef __cplusplus
				s3yyerror( gettxt("uxlibc:81", "syntax error") );
#else
				s3yyerror( gettxt("uxlibc:81", "Syntax error") );
#endif
				goto skip_init;
			s3yyerrlab:
				/*
				** get globals into registers.
				** we have a user generated syntax type error
				*/
				s3yy_pv = s3yypv;
				s3yy_ps = s3yyps;
				s3yy_state = s3yystate;
				s3yynerrs++;
				/* FALLTHRU */
			skip_init:
			case 1:
			case 2:		/* incompletely recovered error */
					/* try again... */
				s3yyerrflag = 3;
				/*
				** find state where "error" is a legal
				** shift action
				*/
				while ( s3yy_ps >= s3yys )
				{
					s3yy_n = s3yypact[ *s3yy_ps ] + S3YYERRCODE;
					if ( s3yy_n >= 0 && s3yy_n < S3YYLAST &&
						s3yychk[s3yyact[s3yy_n]] == S3YYERRCODE)					{
						/*
						** simulate shift of "error"
						*/
						s3yy_state = s3yyact[ s3yy_n ];
						goto s3yy_stack;
					}
					/*
					** current state has no shift on
					** "error", pop stack
					*/
#if S3YYDEBUG
#	define _POP_ "Error recovery pops state %d, uncovers state %d\n"
					if ( s3yydebug )
						printf( _POP_, *s3yy_ps,
							s3yy_ps[-1] );
#	undef _POP_
#endif
					s3yy_ps--;
					s3yy_pv--;
				}
				/*
				** there is no state on stack with "error" as
				** a valid shift.  give up.
				*/
				S3YYABORT;
			case 3:		/* no shift yet; eat a token */
#if S3YYDEBUG
				/*
				** if debugging, look up token in list of
				** pairs.  0 and negative shouldn't occur,
				** but since timing doesn't matter when
				** debugging, it doesn't hurt to leave the
				** tests here.
				*/
				if ( s3yydebug )
				{
					register int s3yy_i;

					printf( "Error recovery discards " );
					if ( s3yychar == 0 )
						printf( "token end-of-file\n" );
					else if ( s3yychar < 0 )
						printf( "token -none-\n" );
					else
					{
						for ( s3yy_i = 0;
							s3yytoks[s3yy_i].t_val >= 0;
							s3yy_i++ )
						{
							if ( s3yytoks[s3yy_i].t_val
								== s3yychar )
							{
								break;
							}
						}
						printf( "token %s\n",
							s3yytoks[s3yy_i].t_name );
					}
				}
#endif /* S3YYDEBUG */
				if ( s3yychar == 0 )	/* reached EOF. quit */
					S3YYABORT;
				s3yychar = -1;
				goto s3yy_newstate;
			}
		}/* end if ( s3yy_n == 0 ) */
		/*
		** reduction by production s3yy_n
		** put stack tops, etc. so things right after switch
		*/
#if S3YYDEBUG
		/*
		** if debugging, print the string that is the user's
		** specification of the reduction which is just about
		** to be done.
		*/
		if ( s3yydebug )
			printf( "Reduce by (%d) \"%s\"\n",
				s3yy_n, s3yyreds[ s3yy_n ] );
#endif
		s3yytmp = s3yy_n;			/* value to switch over */
		s3yypvt = s3yy_pv;			/* $vars top of value stack */
		/*
		** Look in goto table for next state
		** Sorry about using s3yy_state here as temporary
		** register variable, but why not, if it works...
		** If s3yyr2[ s3yy_n ] doesn't have the low order bit
		** set, then there is no action to be done for
		** this reduction.  So, no saving & unsaving of
		** registers done.  The only difference between the
		** code just after the if and the body of the if is
		** the goto s3yy_stack in the body.  This way the test
		** can be made before the choice of what to do is needed.
		*/
		{
			/* length of production doubled with extra bit */
			register int s3yy_len = s3yyr2[ s3yy_n ];

			if ( !( s3yy_len & 01 ) )
			{
				s3yy_len >>= 1;
				s3yyval = ( s3yy_pv -= s3yy_len )[1];	/* $$ = $1 */
				s3yy_state = s3yypgo[ s3yy_n = s3yyr1[ s3yy_n ] ] +
					*( s3yy_ps -= s3yy_len ) + 1;
				if ( s3yy_state >= S3YYLAST ||
					s3yychk[ s3yy_state =
					s3yyact[ s3yy_state ] ] != -s3yy_n )
				{
					s3yy_state = s3yyact[ s3yypgo[ s3yy_n ] ];
				}
				goto s3yy_stack;
			}
			s3yy_len >>= 1;
			s3yyval = ( s3yy_pv -= s3yy_len )[1];	/* $$ = $1 */
			s3yy_state = s3yypgo[ s3yy_n = s3yyr1[ s3yy_n ] ] +
				*( s3yy_ps -= s3yy_len ) + 1;
			if ( s3yy_state >= S3YYLAST ||
				s3yychk[ s3yy_state = s3yyact[ s3yy_state ] ] != -s3yy_n )
			{
				s3yy_state = s3yyact[ s3yypgo[ s3yy_n ] ];
			}
		}
					/* save until reenter driver code */
		s3yystate = s3yy_state;
		s3yyps = s3yy_ps;
		s3yypv = s3yy_pv;
	}
	/*
	** code supplied by user is placed in this switch
	*/
	switch( s3yytmp )
	{
		
case 1:
# line 29 "S3Parser.y"
{ return 0;
			} break;
case 4:
# line 38 "S3Parser.y"
{ no = S3CreateObject(s3yypvt[-1].string,0);
			  S3AddObject(oh,no);
			} break;
case 8:
# line 49 "S3Parser.y"
{ no->id = s3yypvt[-0].num;
			} break;
case 9:
# line 52 "S3Parser.y"
{ no->radius = DblToReal(s3yypvt[-0].num);
			} break;
case 10:
# line 55 "S3Parser.y"
{ no->pos.x = DblToReal(s3yypvt[-2].num);
			  no->pos.y = DblToReal(s3yypvt[-1].num);
			  no->pos.z = DblToReal(s3yypvt[-0].num);
			} break;
case 12:
# line 61 "S3Parser.y"
{ if (no->facec >= MAX_FACES_PER_OBJECT) {
			    parseerror("Object has too many faces",no->name);
			    return 1;
			    }
			  np = no->face[no->facec] =
			    S3Calloc(1,sizeof(S3polygon));
			  np->colour = s3yypvt[-1].num;
			  np->type   = s3yypvt[-1].num;
			} break;
case 13:
# line 71 "S3Parser.y"
{ np->vert = S3MemDup(vert_tmp,np->vertc*sizeof(int));
			  no->facec++;
			} break;
case 14:
# line 75 "S3Parser.y"
{ if (!no->extra_data) {
			    parseerror("No extra data allocated","");
			    return 1;
			    }
			} break;
case 18:
# line 88 "S3Parser.y"
{ if (no->ptc >= MAX_POINTS_PER_OBJECT) {
			    parseerror("Object has too many points",no->name);
			    return 1;
			    }
			  no->pt[no->ptc] = S3Malloc(sizeof(S3point));
			  no->pt[no->ptc]->x = s3yypvt[-2].num;
			  no->pt[no->ptc]->y = s3yypvt[-1].num;
			  no->pt[no->ptc]->z = s3yypvt[-0].num;
			  no->ptc++;
			} break;
case 21:
# line 105 "S3Parser.y"
{ if (np->vertc >= MAX_VERTS_PER_FACE) {
			    parseerror("polygon has too many vertices","");
			    return 1;
			    }
			  if (s3yypvt[-0].num >= no->ptc) {
			    parseerror("Point does not exist",S3Itoa(s3yypvt[-0].num));
			    return 1;
			    }
			  vert_tmp[np->vertc++] = s3yypvt[-0].num;
			} break;
case 23:
# line 120 "S3Parser.y"
{ s3yyval.num = s3yypvt[-0].num; } break;
case 24:
# line 122 "S3Parser.y"
{ switch(s3yypvt[-0].string[0]) {
			  case 'p':
				s3yyval.num = pt_plane; break;
			  case 'w':
				s3yyval.num = 0; break;
			  case 'g':
			 	s3yyval.num = pt_plane | pt_glass; break;
			  case 's':
				s3yyval.num = pt_plane | pt_shaded; break;
			  default:
				s3yyval.num = pt_plane;
				}
			} break;
case 27:
# line 142 "S3Parser.y"
{ evcount = 0;
			} break;
case 28:
# line 145 "S3Parser.y"
{ S3eadd eaf = S3GetEAddFunc();
			  int i;

			  if (!eaf) {
			    parseerror("No extra data addition function given",
				       "");
			    return 1;
			    }
			  eaf(no,s3yypvt[-3].string,evlist,evcount);
			  for(i=0; i<evcount; i++)
			    free(evlist[i]);
			} break;
case 31:
# line 164 "S3Parser.y"
{ if (evcount >= 30) {
			    parseerror("Too many attributes",s3yypvt[-0].string);
			    return 1;
			    }
			  evlist[evcount] = malloc(strlen(s3yypvt[-0].string)+1);
			  strcpy(evlist[evcount],s3yypvt[-0].string);
			  evcount++;
			} break;
	}
	goto s3yystack;		/* reset registers in driver code */
}
